"use strict";
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var React = require("react");
var _ = require("underscore");
var moment = require("moment");
var grid_1 = require("./../../layout/grid");
var classBuilder_1 = require("./../../../utilities/classBuilder");
var DatePickerInput = (function (_super) {
    __extends(DatePickerInput, _super);
    function DatePickerInput(props) {
        _super.call(this, props);
        this.mouseDownOnCalendar = false;
        var todaysDate = moment().startOf('day');
        this.state = { displayedDate: todaysDate.clone(), selectedDate: todaysDate, pickerBodyVisible: false, showOnTop: false, isMobile: false };
        moment.locale(props.locale);
        var seed = Math.random().toFixed(4);
        this.inputElementId = "date-picker-text-input-" + seed;
        this.bodyElementId = "date-picker-body-" + seed;
    }
    DatePickerInput.prototype.onDaySelected = function (date) {
        if (!this.fallsWithinRange(date)) {
            return;
        }
        var newDate = date.clone();
        this.inputElement.value = newDate.format(this.props.format);
        this.setState({ selectedDate: newDate, displayedDate: newDate.clone(), pickerBodyVisible: false });
        if (this.props.onDateChanged) {
            this.props.onDateChanged(newDate.clone());
        }
    };
    DatePickerInput.prototype.isEndOfMonth = function (date) {
        var endOfMonth = date.clone().endOf('month');
        return endOfMonth.isSame(date, 'day');
    };
    DatePickerInput.prototype.fallsWithinRange = function (date) {
        if (this.props.minDate && date.isBefore(this.props.minDate, 'day')) {
            return false;
        }
        if (this.props.maxDate && date.isAfter(this.props.maxDate, 'day')) {
            return false;
        }
        return true;
    };
    DatePickerInput.prototype.getDaysInMonth = function () {
        var days = [];
        var a = this.state.displayedDate.clone().startOf('month').startOf('day');
        var b = a.clone().endOf('month');
        var firstDay = false;
        for (var m = moment(a); m.isBefore(b); m.add(1, 'days')) {
            if (!firstDay) {
                firstDay = true;
                var firstDayIndex = m.weekday();
                for (var i = firstDayIndex; i > 0; i--) {
                    days.push(this.getDayComponent(true, this.onDaySelected.bind(this), m.clone().subtract(i, 'days')));
                }
            }
            days.push(this.getDayComponent(false, this.onDaySelected.bind(this), m.clone()));
            if (this.isEndOfMonth(m)) {
                var lastDayIndex = m.weekday();
                for (var i2 = 1; i2 < 7 - lastDayIndex; i2++) {
                    days.push(this.getDayComponent(true, this.onDaySelected.bind(this), m.clone().add(i2, 'days')));
                }
            }
        }
        return days;
    };
    DatePickerInput.prototype.getDayComponent = function (notInCurrentMonth, dayClicked, date) {
        var d = date.clone();
        var dateWithinRange = this.fallsWithinRange(d);
        var isSelected = d.isSame(this.state.selectedDate, 'day');
        return React.createElement(DatePickerDay, {key: "datepicker_day_" + date.format('DDMMYYYY'), selected: isSelected, withinRange: dateWithinRange, notInCurrentMonth: notInCurrentMonth, dayClicked: dayClicked, date: d});
    };
    DatePickerInput.prototype.changeMonth = function (increment) {
        var _this = this;
        this.setState({ displayedDate: this.state.displayedDate.add(increment, 'months') }, function () {
            _this.shouldShowOnTop();
        });
    };
    DatePickerInput.prototype.checkDate = function (dateString) {
        var m = moment(dateString, this.props.format, false);
        if (m.isValid() && this.fallsWithinRange(m)) {
            var formattedDate = m.format(this.props.format);
            this.inputElement.value = formattedDate;
            this.setState({ selectedDate: m.clone(), displayedDate: m.clone() });
            if (this.props.onDateChanged) {
                this.props.onDateChanged(m.clone());
            }
        }
        else {
            this.inputElement.value = this.state.selectedDate.format(this.props.format);
        }
    };
    DatePickerInput.prototype.componentWillMount = function () {
        if (this.props.date) {
            if (typeof this.props.date === "string") {
                var mDate = moment(this.props.date, "DD/MM/YYYY");
                this.setState({ selectedDate: mDate, displayedDate: mDate.clone() });
            }
            else {
                var pDate = moment(this.props.date).startOf('day');
                pDate.locale(this.props.locale);
                this.setState({ selectedDate: pDate, displayedDate: pDate.clone() });
            }
        }
    };
    DatePickerInput.prototype.componentWillUnmount = function () {
        window.removeEventListener('mousedown', this);
    };
    DatePickerInput.prototype.componentDidMount = function () {
        this.inputElement = document.getElementById(this.inputElementId);
        this.bodyElement = document.getElementById(this.bodyElementId);
        this.checkMobile(window.innerWidth);
        window.addEventListener('mousedown', this);
    };
    DatePickerInput.prototype.checkMobile = function (width) {
        if (width < 500 && !this.state.isMobile) {
            this.setState({ isMobile: true });
        }
        if (width > 500 && this.state.isMobile) {
            this.setState({ isMobile: false });
        }
    };
    DatePickerInput.prototype.handleEvent = function (e) {
        if (this.mouseDownOnCalendar) {
            return;
        }
        this.setState({ pickerBodyVisible: false });
    };
    DatePickerInput.prototype.onInputFocus = function () {
        var _this = this;
        this.setState({ pickerBodyVisible: true }, function () {
            _this.shouldShowOnTop();
        });
    };
    DatePickerInput.prototype.shouldShowOnTop = function () {
        if (window.innerWidth < 480) {
            return;
        }
        var height = this.bodyElement.clientHeight + 25;
        var visibleBottom = (window.innerHeight + window.scrollY);
        var inputRect = this.inputElement.getBoundingClientRect();
        var remainingSpace = visibleBottom - inputRect.bottom;
        if (remainingSpace < height) {
            this.setState({ showOnTop: true });
        }
        else {
            this.setState({ showOnTop: false });
        }
    };
    DatePickerInput.prototype.render = function () {
        var _this = this;
        var weekdays = _.range(0, 7).map(function (n) { return React.createElement("div", {className: "date-picker-week-day", key: "day_name_" + n}, moment().startOf('week').add(n, 'days').format('dd')); });
        var days = this.getDaysInMonth();
        var currentDisplayDate = this.state.displayedDate.format("MMMM - YYYY");
        var classes = classBuilder_1.classNames("date-picker-body", classBuilder_1.cd("date-picker-body-visible", this.state.pickerBodyVisible), classBuilder_1.cd("date-picker-top", this.state.showOnTop));
        var rootClasses = classBuilder_1.classNames("date-picker", this.props.className);
        return (React.createElement("div", {className: rootClasses, onMouseDown: function () { return _this.mouseDownOnCalendar = true; }, onMouseUp: function () { return _this.mouseDownOnCalendar = false; }}, 
            !this.state.isMobile &&
                React.createElement("input", {id: this.inputElementId, type: "text", defaultValue: this.state.selectedDate.format(this.props.format), onBlur: function (e) { return _this.checkDate(e.target.value); }, onFocus: function (e) { return _this.onInputFocus(); }}), 
            this.state.isMobile &&
                React.createElement("div", {className: "fake-input", onClick: function () { return _this.onInputFocus(); }}, this.state.selectedDate.format(this.props.format)), 
            React.createElement("div", {id: this.bodyElementId, className: classes}, 
                React.createElement("div", {className: "date-picker-body-wrapper"}, 
                    React.createElement(grid_1.Grid, {responsive: "none", className: "date-picker-header"}, 
                        React.createElement(grid_1.Row, null, 
                            React.createElement(grid_1.Col, {onClick: function () { return _this.changeMonth(-1); }, fixed: true}, "<"), 
                            React.createElement(grid_1.Col, null, currentDisplayDate), 
                            React.createElement(grid_1.Col, {onClick: function () { return _this.changeMonth(1); }, fixed: true}, ">"))
                    ), 
                    React.createElement("div", {className: "date-picker-days"}, 
                        weekdays, 
                        days))
            )));
    };
    DatePickerInput.defaultProps = {
        format: 'DD/MM/YYYY',
        date: moment().startOf('day'),
        locale: 'en-gb'
    };
    return DatePickerInput;
}(React.Component));
exports.DatePickerInput = DatePickerInput;
var DatePickerDay = (function (_super) {
    __extends(DatePickerDay, _super);
    function DatePickerDay() {
        _super.apply(this, arguments);
    }
    DatePickerDay.prototype.render = function () {
        var _this = this;
        var classes = classBuilder_1.classNames(classBuilder_1.cd('not-in-month', this.props.notInCurrentMonth), classBuilder_1.cd('selected-day', this.props.selected), classBuilder_1.cd("day-disabled", !this.props.withinRange));
        return React.createElement("div", {className: classes, onClick: function () { return _this.props.dayClicked(_this.props.date); }}, this.props.date.format('DD'));
    };
    return DatePickerDay;
}(React.Component));
