"use strict";
(function (SpacingType) {
    SpacingType[SpacingType["Padding"] = 0] = "Padding";
    SpacingType[SpacingType["Margin"] = 1] = "Margin";
})(exports.SpacingType || (exports.SpacingType = {}));
var SpacingType = exports.SpacingType;
(function (ColorType) {
    ColorType[ColorType["Foreground"] = 0] = "Foreground";
    ColorType[ColorType["Background"] = 1] = "Background";
})(exports.ColorType || (exports.ColorType = {}));
var ColorType = exports.ColorType;
var LayoutHelpers = (function () {
    function LayoutHelpers() {
    }
    LayoutHelpers.GetBackgroundImage = function (background) {
        if (!background) {
            return null;
        }
        var style = {};
        if (background.url) {
            style.backgroundImage = "url(" + background.url + ")";
        }
        return style;
    };
    LayoutHelpers.HandleLayoutClasses = function (margins, paddings, contentCentering, textAlignment) {
        var ms = this.GetSpacings(margins, SpacingType.Margin);
        var ps = this.GetSpacings(paddings, SpacingType.Padding);
        var al = this.GetAlignment(contentCentering);
        var tAl = this.GetTextAlignment(textAlignment);
        return [ms, ps, al].filter(function (cl) { return cl != null; }).join(" ");
    };
    LayoutHelpers.HandleResponsivenessClasses = function (responsiveness) {
        if (responsiveness === "both") {
            return "responsive-phone responsive-tablet";
        }
        return "responsive-" + responsiveness;
    };
    LayoutHelpers.HandleDisplayClasses = function (background, foreground) {
        var fgc = foreground ? this.GetColorClass(foreground.color, ColorType.Foreground) : null;
        var bgc = background ? this.GetColorClass(background.color, ColorType.Background) : null;
        return [bgc, fgc].filter(function (cl) { return cl != null; }).join(" ");
    };
    LayoutHelpers.HandleDisplayStyles = function (background, foreground) {
        var bg = this.GetBackgroundImage(background);
        return bg;
    };
    LayoutHelpers.GetMargin = function (margins) {
        return this.GetSpacings(margins, SpacingType.Margin);
    };
    LayoutHelpers.GetPadding = function (paddings) {
        return this.GetSpacings(paddings, SpacingType.Padding);
    };
    LayoutHelpers.GetTextAlignment = function (textAlignment) {
        return "t-al-" + textAlignment;
    };
    LayoutHelpers.GetAlignment = function (centerContent) {
        if (!centerContent) {
            return null;
        }
        if (typeof centerContent === "string") {
            return "align-content-center-both";
        }
        var classes = [];
        classes.push(centerContent.horizontal ? LayoutHelpers.GetHorizontalAlignment(centerContent.horizontal) : null);
        classes.push(centerContent.vertical ? LayoutHelpers.GetVerticalAlignment(centerContent.vertical) : null);
        return classes.filter(function (c) { return c != null; }).join(" ");
    };
    LayoutHelpers.GetHorizontalAlignment = function (alignment) {
        return "align-content-horizontal-" + alignment;
    };
    LayoutHelpers.GetVerticalAlignment = function (alignment) {
        return "align-content-vertical-" + alignment;
    };
    LayoutHelpers.GetSpacings = function (spacing, spacingType) {
        if (typeof spacing === "string") {
            return this.GetSpacingClass(spacing, "all", spacingType);
        }
        else {
            var classes = [];
            if (!spacing) {
                return null;
            }
            classes.push(LayoutHelpers.GetSpacingClass(spacing.top, "top", spacingType));
            classes.push(LayoutHelpers.GetSpacingClass(spacing.bottom, "bottom", spacingType));
            classes.push(LayoutHelpers.GetSpacingClass(spacing.left, "left", spacingType));
            classes.push(LayoutHelpers.GetSpacingClass(spacing.right, "right", spacingType));
            return classes.filter(function (c) { return c != undefined; }).join(" ");
        }
    };
    LayoutHelpers.GetColorClass = function (color, colorType) {
        if (colorType === ColorType.Foreground) {
            return "fg-" + color;
        }
        else if (colorType === ColorType.Background) {
            return "bg-" + color;
        }
        else {
            throw new Error("Invalid color type specified");
        }
    };
    LayoutHelpers.GetSpacingClass = function (size, side, type) {
        if (!size) {
            return null;
        }
        if (side === "all") {
            return type === SpacingType.Margin ? "m-" + size : "p-" + size;
        }
        return type === SpacingType.Margin ? "m-" + side + "-" + size : "p-" + side + "-" + size;
    };
    return LayoutHelpers;
}());
exports.LayoutHelpers = LayoutHelpers;
